/*
  Author:  Jonathan Bondzie, Dr. Chuck Yeung
           Lennard Gas Simulation Source

	   Copyright (C) 1999  Penn State University

	   This program is free software; you can redistribute it and/or
	   modify it under the terms of the GNU General Public License
	   as published by the Free Software Foundation;
	   
	   This program is distributed in the hope that it will be useful,
	   but WITHOUT ANY WARRANTY; without even the implied warranty of
	   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	   GNU General Public License for more details.
	   
	   You should have received a copy of the GNU General Public License
	   along with this program; if not, write to the Free Software
	   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  
	   02111-1307, USA.

	   email jab341@psu.edu
*/

#include	"motifR.h"
#include	"Headers.h" 


/* simple version of graphics setup routine */

extern void scaleKE(Widget widget, XtPointer client_data, XtPointer call_data);
Widgets* wptr = new Widgets;    /*an instance to the Widget class */


/************************************************************************
 *                                                                      *
 *    these are the functions that create all the widgets needed under  *
 *    parameter widget.                                                 *
 *                                                                      *
 ************************************************************************/


/* prototypes for the callback for the prompt dailog box */   

void ok_callback(Widget parent,XtPointer client_data,XtPointer call_data);
void form_callback(Widget paretn,XtPointer client_data,XtPointer call_data);
void cancel_callback(Widget parent,XtPointer client_data,XtPointer call_data);
void help_callback(Widget parent,XtPointer client_data,XtPointer call_data);
void KeCallback(Widget widget,XtPointer client_data,XtPointer call_data);


/* callback procedure for changing the value of the kinetic enery */
Widget dgr_text;
void temp_callback(Widget widget,XtPointer client_data,XtPointer call_data)
{
  
  
  Widget dgr_ke;
  Arg args[4];
  
  /* Create the Pop_up Shell as a continer widget */
	
  dgr_ke = XmCreateFormDialog((*wptr).dgr_mainwindow,"change KE",args,(Cardinal) 0);
  
  /* create a text widget to hold the change in kinetic energy */
  
  dgr_text = XtVaCreateManagedWidget("kinetic_text",
				     xmTextWidgetClass, 
				     dgr_ke,
				     XmNeditMode,
				     XmSINGLE_LINE_EDIT,
				     NULL);
  XtAddCallback(dgr_text,XmNactivateCallback,(XtCallbackProc)KeCallback,(XtPointer)client_data);
 	
  call_data = call_data;
  widget = widget;
  XtManageChild(dgr_ke);
}

/* callback routine for changing the kinetic energy */

void KeCallback(Widget widget,XtPointer client_data,XtPointer call_data)
{
  char* string;
  float number =0;
  Variables* ptr = (Variables*)client_data;

  string = GetTextString(dgr_text);
  number = atof(string);
   
  for(int i=0;i<(*ptr).nparticles;i++)
  {
    (*ptr).vx[i] = (*ptr).vx[i]*sqrt(number);
    (*ptr).vy[i] = (*ptr).vy[i]*sqrt(number);
  }
  widget = widget;
  call_data = call_data;
}


/* I need a link between SetupInitialGraphics,FormCreationFunctiona and Drawcircles 
   in order to have the true address of Widgets class. ReturnAddress() is the function
   that has the true address of the Widgets class address. This function will be prototyped
   in the Headers.h file and decleared in SetupInitialGraphics. this way, when the function
   is called it will have the address of the Widgets class                                 */

Widgets* ReturnClassAddress(void)
{
  return wptr;
}


/* 
   geometry variables -- set static so that it can be used byXtManageChild(dgr_initial_conditions);
   all the different subroutines   

*/


/* default resources -- can be overwritten by resource file */

static String fallback_resources[] = {
	"*drawarea.width: 600",
	"*drawarea.height: 600",
	"*.drawarea.background:  lightblue",
	"*.drawarea.foreground: red",
	"*time.*.columns: 9",
	"*form.*.columns: 5",
	"*physical.*.columns:9",
	"*Radio.*.selectColor:red",
	NULL };

/***********************************************************************/
/*	this routine sets up all the graphics for the

 *	 -- the graphics tree is given below
 *
 *	dgr_parent_widget -> dgr_mainwindow
 *	 	dgr_mainwindow	->  dgr_menubar -> 
 *		       		->  dgr_command_window ->	      
 *			       	->  dgr_drawing_area ->		      
 *
 *		dgr_command_window -> dgr_time_window
 *					-> dgr_time_text
 *				   -> dgr_wall_window
 *					-> dgr_wall_text
 *				   -> dgr_message_window
 *
 *		
 ***********************************************************************
 */

XtAppContext SetupInitialGraphics(
	int argc,
	char** argv,
	Variables* ptr
	)
{

  /* this makes the storage placement prettier */
  
  Widget  dgr_command_window;  	/* window for commands and text */
  Widget  dgr_time_window;      /* holds times */	
  Widget  dgr_menubar;		/* menu bar widget */
  Widget  dgr_action_menu;	/* action menu */
  Widget  dgr_help_menu;      	/* help menu */
  Widget  dgr_kinetic_label;    /* widget for labeling the kinetic energy */
  Widget  dgr_potential_label;
  Widget  dgr_total_label;
  Widget  dgr_form_pushbutton;    /* pushbutton widget that conects the form and menubar */
  Widget  dgr_initial_conditions; /* actual conection between menubar and other widget */
  Widget  dgr_physical;           /* holds the physical quantities in the simulation */
  Widget  dgr_itlabel;            /* instantanuos temperature label */
  Widget  dgr_iplabel; 
  Widget  dgr_ke_change;          /* widget that holds the kinetic energy change */
  
  Widget	dgr_parent_widget;/* top parent widget */
  XtAppContext dgr_app_context;	  /* infinite loop waits for events */
  
  Arg	args[20];		  /* dummy used in arguments */
  
  Dimension       width, height;  /* width and height of drawing area */
  
  static FptrWptr endtimeStruct;  /* structure used in continue buttons 
				   *	this structure must be static
				   *	if the pushbuttons are called
				   *	from other routines 
				   */
  
  Cardinal n=0;                    /* n is an argument */
  
  /* create parent widget with title and unmapped at manage */
  
  dgr_parent_widget =
    XtAppInitialize(&dgr_app_context, "SANDrc",
		    (XrmOptionDescList)NULL, 0,  &argc, argv,
		    fallback_resources,
		    args, n);
  
  /* Create the mainwindow  -- mainwindow is managed after all child
   *      processes are set up
   *	mainwindow is created so that the different sections
   *	have a separator space
   */
  
  (*wptr).dgr_mainwindow = XmCreateMainWindow(dgr_parent_widget,
				      "main", args, (Cardinal) 0);
  XtVaSetValues((*wptr).dgr_mainwindow, XmNshowSeparator, True, NULL); 
  
  (*wptr).dgr_widget = XmCreatePanedWindow((*wptr).dgr_mainwindow,"command", args, (Cardinal)n);
  
  
  /********************************* menu section ****************************/
  
  /* create menubar */
  n=0;
  dgr_menubar = XmCreateMenuBar((*wptr).dgr_mainwindow, "menubar", args,n);
  XtManageChild(dgr_menubar);
  
  /* create various menus */
  
  /* create the main action menu --
   *	note that label and accelerator keys are set in
   *	resource action following recommendation of motif book
   *	remember that tha name in the resource action is not
   *	the Widget variable but the variable name passed to
   *	the create functions
   */
  
  dgr_action_menu = CreatePulldownMenu(dgr_menubar, "action");
  
  /* store the structure used in the continue buttons 
   * 	you have to store things in structures if your callback 
   * 	uses more than one function since the event dispatcher just
   *	passes one pointer to the callback function
   * 	remember also that if you want to update something you
   *	in your callback you will have to pass things as pointers
   */
  
  endtimeStruct.floatptr = &(*ptr).end_time;
  endtimeStruct.widgptr = &(*wptr).dgr_endtime_text;
  
  /* create button to increase endtime by CONTINUE_TIME */
  
  CreatePushbutton(dgr_action_menu, "continue", continueCB,(XtPointer) &endtimeStruct);
  
  /* create button to double endtime */
  
  CreatePushbutton(dgr_action_menu, "double", doubleCB,(XtPointer) &endtimeStruct);
  XtVaCreateManagedWidget("sep", xmSeparatorWidgetClass,dgr_action_menu, NULL);
  
  /* exit button */
  
  XtVaCreateManagedWidget("sep", xmSeparatorWidgetClass,dgr_action_menu, NULL);
  CreatePushbutton(dgr_action_menu, "exit", exitCB, "Exit");
  
  /* create the buttons for changing the kinetic energies */
  
  n=0;
  dgr_ke_change = XmCreatePushButton(dgr_action_menu,"change KE",args,n);
	
  XtAddCallback(dgr_ke_change,XmNactivateCallback,(XtCallbackProc)temp_callback,(XtPointer)&(*ptr));
  
  XtManageChild(dgr_ke_change);
  
  /* create a widget to hold the form widget on the menubar */
  
  dgr_initial_conditions = CreatePulldownMenu(dgr_menubar,"parameters");

  /* I need the address of the class before the for widget is created.
     I need to fill in the initial values for the variables.          */

  Function(ptr);

  /* create the pushbutton and its callback for the form widget */
  
  dgr_form_pushbutton = XmCreatePushButton(dgr_initial_conditions,"Parameters",args,n);
  
  /* create the form widget now to avoid all complications */
  n =0;
  XtSetArg(args[n],XmNautoUnmanage,False);n++;
  (*wptr).dgr_form =  XmCreateFormDialog((*wptr).dgr_widget,"form",args,(Cardinal) n); 
  
  /* call the function to create the form children */
  
  CallFormChildren();
  XtAddCallback(dgr_form_pushbutton,XmNactivateCallback,(XtCallbackProc)FormPushButtonCallback,
		(XtPointer)NULL);
  XtManageChild(dgr_form_pushbutton);
  
  /* managet the whole sub-menu containing these widgets */
  
  XtManageChild(dgr_initial_conditions);
    
  /* help menu */
  
  dgr_help_menu = CreateHelpMenu(dgr_menubar, "help");
  XtVaCreateManagedWidget("search (man lennard)",
			  xmLabelWidgetClass, dgr_help_menu, NULL);
  
  
  /********************************************************************
    Beginning of the main window widget creations
    ********************************************************************/
    
  /**********Command window widget creation***********************/
    
  /* make the command window in which data is placed */
  
  dgr_command_window = XmCreatePanedWindow((*wptr).dgr_mainwindow,
					   "command", args, (Cardinal)0);
  
  /*************Time window widget creations****************/
  
  
  /* makes the time window in which times are placed */
  
  dgr_time_window = XmCreateRowColumn(dgr_command_window,
				      "time", args, (Cardinal)0);
  XtVaSetValues(dgr_time_window, XmNorientation, XmHORIZONTAL,
		XmNpacking, XmPACK_TIGHT, NULL);
  
  /* time text window */
  
  (*wptr).dgr_time_text = createTableFloat(dgr_time_window,"readonly", "t: ", 0.);
  XmTextSetEditable((*wptr).dgr_time_text, False);
  
  /* setup endtime text and make it editable */
  
  (*wptr).dgr_endtime_text= createTableFloat(dgr_time_window,"readwrite", " end:", (*ptr).end_time);
  XtAddCallback((*wptr).dgr_endtime_text, XmNactivateCallback,
                (XtCallbackProc) newfloatCB, (XtPointer) &(*ptr).end_time);
  
  XtManageChild(dgr_time_window);
    
  /****************end of time window widgets*****************/
  
  
  /***************Beginning of the energy window widget creation***********/
    
  /* make the energy_window the parent window for the energy calculations */
  
  XtSetArg(args[n],XmNorientation, XmHORIZONTAL);
  n++;
  (*wptr).dgr_energy_window = XmCreateRowColumn(dgr_command_window,"time", args,n);
    
  /* create the kinetic energy window*/
  
  dgr_kinetic_label  = XmCreateLabel((*wptr).dgr_energy_window,"KE",args,n);
  XtManageChild(dgr_kinetic_label);
  (*wptr).dgr_kinetic_window = XmCreateText((*wptr).dgr_energy_window,"KEtext",args,n);
  XtManageChild((*wptr).dgr_kinetic_window);
    
  /* create the potential energy window */
  
  dgr_potential_label =  XmCreateLabel((*wptr).dgr_energy_window,"PE",args,n);
  XtManageChild(dgr_potential_label);
  (*wptr).dgr_potential_window = XmCreateText((*wptr).dgr_energy_window,"PEtext",args,n);
  XtManageChild((*wptr).dgr_potential_window);
  
  /* create the total energy window */
  
  dgr_total_label = XmCreateLabel((*wptr).dgr_energy_window,"TE",args,n);
  XtManageChild(dgr_total_label);
  (*wptr).dgr_total_energy_window = XmCreateText((*wptr).dgr_energy_window,"TEtext",args,n);
  XtManageChild((*wptr).dgr_total_energy_window);
  
  /* manage the energy window */
  
  XtManageChild((*wptr).dgr_energy_window);
    
  /**************** end of the energy window creation widget *****************/
  
  
  /**********************Beginning of the physical quatity widget*********** */
  
  dgr_physical = XmCreateRowColumn(dgr_command_window,"physical", args, (Cardinal)0);
  
  XtVaSetValues(dgr_physical, XmNorientation, XmHORIZONTAL,XmNpacking, XmPACK_TIGHT, NULL);
  
  /* create the instant pressure */
    
  n=0;
  dgr_iplabel = XmCreateLabel(dgr_physical,"Pressure",args,n);
  XtManageChild(dgr_iplabel);
  (*wptr).dgr_ipressure = XmCreateText(dgr_physical,"Pressure",args,n);
  XtManageChild((*wptr).dgr_ipressure);
  
  
  /* create the instant temperature */
  
  n=0;
  dgr_itlabel = XmCreateLabel(dgr_physical,"Temperature(K)",args,n);
  XtManageChild(dgr_itlabel);
  (*wptr).dgr_itemp = XmCreateText(dgr_physical,"Temp",args,n);
  XtManageChild((*wptr).dgr_itemp);
  XtManageChild(dgr_physical);	
  

  /***********************end of dgr_physical widget***********************/
  
  
  /* make the message window */ 
  
  (*wptr).dgr_message_window = XmCreateScrolledText(dgr_command_window,"message",args,(Cardinal)0);
  XtManageChild((*wptr).dgr_message_window);
    
  /* manage the whole command window */
  
  XtManageChild(dgr_command_window);
    
  /******************end of the command window widget *********************/
  
  
  /* makes the drawing area in which the balls are placed */
  
  (*wptr).dgr_drawing_area = CreateDrawingArea((*wptr).dgr_mainwindow,
				       "drawarea",drawCB,(XtPointer) &(*wptr).dgr_gc);
  
  /* scale drawing area so it has correct aspect ratio --
   * 800 is maximum height of screen -- make it a little
   * smaller to give so it doesn't fill whole thing
   */
  
  XtVaGetValues((*wptr).dgr_drawing_area, XmNwidth, &width, NULL);
  height = (Dimension) ( width * (*ptr).ly/(*ptr).lx );
  if( (int) height > 900 ) height = (Dimension) 900;
  XtVaSetValues((*wptr).dgr_drawing_area,XmNwidth,width, XmNheight, height,NULL);
  XmMainWindowSetAreas((*wptr).dgr_mainwindow, dgr_menubar,dgr_command_window, (Widget) NULL, 
		       (Widget) NULL,(*wptr).dgr_drawing_area);
  
  /* keep up the widget order */
  
  XtManageChild((*wptr).dgr_widget);
  XtManageChild((*wptr).dgr_mainwindow);
  XtRealizeWidget(dgr_parent_widget);
  
  /* create GC (graphics context)  -- you have to realize the
   * parent before creating the graphics content */
  
  (*wptr).dgr_gc = XCreateGC((Display*)XtDisplay((*wptr).dgr_drawing_area),
		     XtWindow((*wptr).dgr_drawing_area), 0L, (XGCValues*) NULL);
  
  XtMapWidget(dgr_parent_widget);
  XtManageChild((*wptr).dgr_form);
  return dgr_app_context;
  
}


